package com.swagger.rest.controllers

import com.swagger.rest.models.Enum
import com.swagger.rest.models.MemberInput
import com.swagger.rest.models.MemberOutput
import com.swagger.rest.models.ProjectMember
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("/api/v1")
class MemberController(
    private val memberRepository: MemberRepository,
    private val projectRepository: ProjectRepository,
    private val userRepository: UserRepository
) {

    @GetMapping("/projects/{id}/member")
    fun getMemberByProject(@PathVariable("id") id: Long): Any {
        val memberData = memberRepository.findByProject(id.toString())
        val output = memberData.map {
            MemberOutput(
                id = it.id,
                project = it.project!!.description,
                user = it.user!!.name,
                role = Enum.Member.values()[it.role]
            )
        }
        return if (memberData.isNotEmpty()) {
            ResponseEntity(output, HttpStatus.OK)
        } else {
            arrayOf<String>()
        }
    }

    @GetMapping("/projects/member/{id}")
    fun getMemberProject(@PathVariable("id") id: Long): Any {
        val memberData = memberRepository.findById(id)
        val output = memberData.map {
            MemberOutput(
                id = it.id,
                project = it.project!!.description,
                user = it.user!!.name,
                role = Enum.Member.values()[it.role]
            )
        }
        return if (memberData.isPresent) {
            ResponseEntity<MemberOutput>(output.get(), HttpStatus.OK)
        } else {
            arrayOf<String>()
        }
    }

    @DeleteMapping("/projects/member/{id}")
    fun deleteProjectMember(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        return try {
            val find = memberRepository.findById(id)
            if (find.isPresent) {
                val validOwner = projectRepository.validOwner(find.get().project!!.id.toString(), userId.id.toString())
                val validAdmin =
                    memberRepository.validRole(find.get().project!!.id.toString(), userId.id.toString(), "2")
                val targetNotAdmin = memberRepository.targetRole(
                    find.get().project!!.id.toString(), find.get().user!!.id.toString(), find.get().role.toString()
                )
                if (validOwner > 0 || (validAdmin > 0 && targetNotAdmin > 0)) {
                    memberRepository.deleteById(id)
                    ResponseEntity(HttpStatus.OK)
                } else {
                    ResponseEntity(HttpStatus.FORBIDDEN)
                }
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PostMapping("/projects/{id}/member")
    fun addProjectMember(
        @PathVariable("id") id: Long, @RequestBody member: MemberInput
    ): Any {
        return try {
            val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val found = projectRepository.findById(id)
            val user = userRepository.findById(member.user_id!!.toLong())
            val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
            val validAdmin = memberRepository.validRole(id.toString(), userId.id.toString(), "2")
            if (found.isPresent && user.isPresent) {
                if (validOwner > 0 || validAdmin > 0) {
                    if (enumValues<Enum.Member>().any { it.name == member.role!!.uppercase() }) {
                        val role = Enum.Member.valueOf(member.role!!.uppercase()).ordinal
                        val dup =
                            memberRepository.findDup(id.toString(), role.toString(), member.user_id.toString()).size
                        if (dup > 0) {//duplicate
                            ResponseEntity<ProjectMember>(HttpStatus.CONFLICT)
                        } else {
                            val saveMember = ProjectMember()
                            saveMember.project = found.get()
                            saveMember.user = user.get()
                            saveMember.role = role
                            val save = memberRepository.save(saveMember)
                            if (listOf(save).isNotEmpty()) {
                                val output = MemberOutput(
                                    id = save.id,
                                    project = save.project!!.description,
                                    user = save.user!!.name,
                                    role = Enum.Member.values()[save.role]
                                )
                                ResponseEntity(output, HttpStatus.CREATED)
                            } else {
                                ResponseEntity(null, HttpStatus.INTERNAL_SERVER_ERROR)
                            }
                        }
                    } else {// invalid
                        ResponseEntity<ProjectMember>(HttpStatus.BAD_REQUEST)
                    }
                } else {
                    ResponseEntity<ProjectMember>(HttpStatus.FORBIDDEN)
                }
            } else {
                ResponseEntity<ProjectMember>(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity<ProjectMember>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}