package com.swagger.rest.controllers

import com.swagger.rest.models.Project
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class ProjectController(
    private val projectRepository: ProjectRepository,
    private val platformRepository: PlatformRepository,
    private val userRepository: UserRepository,
    private val memberRepository: MemberRepository
) {

    @GetMapping("/projects")
    fun getProject(@RequestParam(required = false) name: String?): Any {
        return try {
            val projects: List<Project> =
                if (name == null) projectRepository.findAll() else projectRepository.findByNameContaining(
                    name
                )
            if (projects.isEmpty()) {
                arrayOf<String>()
            } else ResponseEntity<List<Project>?>(projects, HttpStatus.OK)
        } catch (e: Exception) {
            ResponseEntity<List<Project>?>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/projects/{id}")
    fun getProjectById(@PathVariable("id") id: Long): ResponseEntity<Project> {
        val projectData: Optional<Project?> = projectRepository.findById(id)
        return if (projectData.isPresent) {
            ResponseEntity<Project>(projectData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<Project>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/projects")
    fun addProject(@RequestBody project: Project): ResponseEntity<Project> {
        return try {
            val found = projectRepository.countByName(project.name)
            if (project.name.isNotBlank()) {
                if (project.name.length > 100 || project.description!!.length > 255) {//too long
                    ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {//duplicate
                    ResponseEntity<Project>(HttpStatus.CONFLICT)
                } else {
                    val username: String = SecurityContextHolder.getContext().authentication.name
                    val userLogin = userRepository.getUserByUsername(username)
                    project.name = project.name.trim()
                    project.owner = userLogin
                    val saveProject: Project = projectRepository.save(project)
                    ResponseEntity<Project>(saveProject, HttpStatus.CREATED)
                }
            } else {//name invalid
                ResponseEntity<Project>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<Project>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/projects/{id}")
    fun updateProjectById(@PathVariable("id") id: Long, @RequestBody project: Project): ResponseEntity<out Any?> {
        val projectData = projectRepository.findById(id)
        val found = projectRepository.countByName(project.name)
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
        val validAdmin = memberRepository.validRole(id.toString(), userId.id.toString(), "2")
        return if (projectData.isPresent) {
            if (validOwner > 0 || validAdmin > 0) {
                if (project.name.isNotBlank()) {
                    if (project.name.length > 100 || project.description!!.length > 255) {//too long
                        ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
                    } else {
                        if ((project.name == projectData.get().name && found > 0) || (project.name !== projectData.get().name && found == 0)) {
                            val saveProject = projectData.get()
                            saveProject.name = project.name.trim()
                            saveProject.description = project.description
                            ResponseEntity<Any?>(projectRepository.save(saveProject), HttpStatus.OK)
                        } else {//duplicate
                            ResponseEntity<Project>(HttpStatus.CONFLICT)
                        }
                    }
                } else {//name invalid
                    ResponseEntity<Project?>(HttpStatus.BAD_REQUEST)
                }
            } else {
                ResponseEntity<Project>(HttpStatus.FORBIDDEN)
            }
        } else {
            ResponseEntity<Project?>(HttpStatus.NOT_FOUND)
        }
    }

    @DeleteMapping("/projects/{id}")
    fun deleteProject(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = projectRepository.findById(id)
        val used = platformRepository.findByProject(id.toString()).size
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
        return try {
            if (validOwner > 0) {
                if (used > 0) {//child used in transaction
                    ResponseEntity(HttpStatus.RESET_CONTENT)
                } else if (find.isPresent) {
                    projectRepository.deleteById(id)
                    ResponseEntity(HttpStatus.OK)
                } else {//project not found
                    ResponseEntity(HttpStatus.NOT_FOUND)
                }
            } else {
                ResponseEntity(HttpStatus.FORBIDDEN)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

}